"""
Validating params passed to the commands
"""

import click

def validate_option_exists(ctx, param, value):
    """
    A callback to be used to validate a Click option. Raises a
    click.BadParameter exception if the value is not set or if
    the property name does not exist in the config file.
    This should be used when we want a required optional, but do not want an
    argument. Arguments are ordered and do not come with flags making them
    unwieldy at times, particularly when there is a default. If there are two
    argument, bar and foo in that order, and bar has a default the default
    is useless since if only one parameter is specified Click will fail with
    too few arguments. There is no way with Click arguments to have a flag to
    indicate that the value passed in is meant for foo and not bar. If we want
    this sort of functionality then we are stuck with Click optionals which are
    always optional out of the box.
    """
    if not value and param.name not in ctx.obj.keys():
        # Let the user know if there is an environment variable for this param
        if param.envvar:
            raise click.BadParameter(
                ('Option is required and must be specified via '
                 'the command line or using the environment variable "{}".'.
                 format(param.envvar)))
        else:
            raise click.BadParameter(('Option is required and must be '
                                      'specified via the command line.'))
    return value


class MutuallyExclusiveOption(click.Option):
    """
    A Click Option type that is mutually exclusive with another option. Click
    does not support mutually exclusive options out of the box.
    Example usage:
    @click.command()
    @click.option(
        '-a',
        cls=click_util.MutuallyExclusiveOption,
        is_flag=True,
        mutually_exclusive=['b'])
    @click.option(
        '-b',
        cls=click_util.MutuallyExclusiveOption,
        default='123',
        mutually_exclusive=['a'])
    def command(a, b):
        # do stuff
    All other default Click options can still be specified.
    Two examples of when this would be useful:
    1) Providing quiet and verbose flags. Neither or one, but not both can be
    specified.
    2) From the Dephix SDK's upload command: the commands provides a --plugin
    option that points to an already built artifact. It also provides a -b flag
    to build the plugin before it is uploaded as a convenience. These two
    should not be specified together.
    """
    def __init__(self, *args, **kwargs):
        self.mutually_exclusive = set(kwargs.pop('mutually_exclusive', []))
        super(MutuallyExclusiveOption, self).__init__(*args, **kwargs)

    def handle_parse_result(self, ctx, opts, args):
        if self.mutually_exclusive.intersection(opts) and self.name in opts:
            raise click.UsageError(
                '"{}" is mutually exclusive with argument(s) "{}".'.format(
                    self.name, ', '.join(self.mutually_exclusive)))

        return super(MutuallyExclusiveOption,
                     self).handle_parse_result(ctx, opts, args)


class PasswordPromptIf(click.Option):
    """
    Remove the need for a prompt if a parameter is already specified via the
    configuration file. This is done by building a custom class derived from
    click.Option and overriding click.Option.handle_parse_result().
    """
    def __init__(self, *args, **kwargs):
        super(PasswordPromptIf, self).__init__(*args, **kwargs)

    def handle_parse_result(self, ctx, opts, args):
        # remove prompt if password exists in configuration file
        if 'password' in ctx.obj.keys():
            self.prompt = None

        return super(PasswordPromptIf,
                     self).handle_parse_result(ctx, opts, args)


#
# Copyright (c) 2021 by Delphix. All rights reserved.
#

import logging

import click

logger = logging.getLogger(__name__)


def logging_est(logfile_path, debug=False):
    """
    Establish Logging

    :param logfile_path: path to the logfile. Default: current directory.
    :type logfile_path: str
    :param debug: Set debug mode on (True) or off (False).
    :type debug: bool
    """
    logging.basicConfig(
        filename=logfile_path,
        format="%(levelname)s:%(asctime)s:%(message)s",
        level=logging.INFO,
        datefmt="%Y-%m-%d %H:%M:%S",
    )
    logger = logging.getLogger()
    if debug is True:
        logger.setLevel(20)
        print_info("Debug Logging is enabled.")
    print_debug("Logging Started")


def print_debug(print_obj):
    """
    Call this function with a log message to prefix the message with DEBUG
    :param print_obj: Object to print to logfile and stdout
    :type print_obj: type depends on objecting being passed. Typically str
    """

    # Uncomment below line if want more detailed logs on console.
    # It would be better if we can provide an option to user to send debug flag.

    #click.secho(f"DEBUG: {str(print_obj)}", fg="green")
    logger.debug(str(print_obj))


def print_info(print_obj):
    """
    Call this function with a log message to prefix the message with INFO
    :param print_obj: Object to print to logfile and stdout
    :type print_obj: type depends on objecting being passed. Typically str
    """
    click.secho(f"INFO: {print_obj}")
    logger.info(str(print_obj))


def print_warning(print_obj):
    """
    Call this function with a log message to prefix the message with INFO
    :param print_obj: Object to print to logfile and stdout
    :type print_obj: type depends on objecting being passed. Typically str
    """
    click.secho(f"WARN: {print_obj}", fg="yellow", bg="white")
    logger.warning(str(print_obj))


def print_exception(print_obj,ref="dxi-generic"):
    """
    Call this function with a log message to prefix the message with EXCEPTION
    :param print_obj: Object to print to logfile and stdout
    :type print_obj: type depends on objectxing being passed. Typically str

    """
    err_msg = "EXCEPTION: " \
              "{module_ref} :{err}".format(module_ref=ref,err=str(print_obj))
    click.secho(err_msg, fg="red")
    logger.error(" %s" % (err_msg))



#
# Copyright (c) 2021 by Delphix. All rights reserved.
#

import click
from dxi._lib import click_util
from dxi._lib.util import boolean_based_system_exit
from dxi.snapshot.dxi_snapshot import DXISnapshot
from dxi.snapshot.dxi_snapshot import SnapshotConstants


@click.group(
    short_help="\b\nPerform Snapshot Operations.\n"
               "[create]"
)
def snapshot():
    """
    Dsource, VDB and vFile Snapshot operations
    """
    pass


@snapshot.command()
@click.option(
    "--db-name",
    cls=click_util.MutuallyExclusiveOption,
    default=SnapshotConstants.NAME,
    mutually_exclusive=["group", "all_dbs"],
    help="\b\nName of the dataset to create the snapshot.\n"
    "[Required if taking snapshot of a single database]",
)
@click.option(
    "--group",
    cls=click_util.MutuallyExclusiveOption,
    mutually_exclusive=["name", "all_dbs"],
    default=SnapshotConstants.GROUP,
    help="\b\nName of group in Delphix to refresh against.\n"
    "If provided, Delphix will take snapshots of \n"
    "all datasets in this group.\n"
    "[Required if taking snapshot of a all databases in a group]",
)
@click.option(
    "--all-dbs",
    cls=click_util.MutuallyExclusiveOption,
    mutually_exclusive=["name", "group"],
    is_flag=True,
    default=SnapshotConstants.ALL_DBS,
    help="\b\nTake a snapshot of all datasets on an engine.\n"
    "[Required if taking snapshot of a all databases on an engine.]",
)
@click.option(
    "--use-recent-backup",
    help="\b\nSnapshot using the most recent available backup.\n"
    "MSSQL and ASE Only.",
    default=SnapshotConstants.USE_BACKUP,
    is_flag=True,
)
@click.option(
    "--create-backup",
    help="\b\nTake a new backup to create a snapshot\n"
    "Full Backup for ASE OR Copy-Only Backup for MSSQL.",
    default=SnapshotConstants.CREATE_BACKUP,
    is_flag=True,
)
@click.option(
    "--backup-file",
    help="\b\nName of a specific backup to use for snapshot.\n"
    "For ASE, provide backup file(s) name.\n"
    "For MSSQL, provide backup uuid.",
    default=SnapshotConstants.BCK_FILE,
)
@click.option(
    "--engine",
    default=SnapshotConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread\n"
    "[Default: {}]".format(SnapshotConstants.SINGLE_THREAD),
    default=SnapshotConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=SnapshotConstants.PARALLEL,
)
@click.option(
    "--poll",
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(SnapshotConstants.POLL),
    default=SnapshotConstants.POLL,
)
@click.option(
    "--config",
    help="Full path to the dxtools.conf file with filename.",
    default=SnapshotConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="Full path to the logs folder.",
    default=SnapshotConstants.LOG_FILE_PATH,
)
def create(
    group,
    db_name,
    all_dbs,
    engine,
    single_thread,
    backup_file,
    use_recent_backup,
    create_backup,
    parallel,
    poll,
    config,
    log_file_path,
):
    """
    Snapshot a Delphix dSource, VDB or vFile.
    """
    snapshot_obj = DXISnapshot(
        engine=engine,
        group=group,
        db_name=db_name,
        all_dbs=all_dbs,
        parallel=parallel,
        poll=poll,
        config=config,
        log_file_path=log_file_path,
        single_thread=single_thread,
    )
    boolean_based_system_exit(
        snapshot_obj.create_snapshot(
            use_recent_backup=use_recent_backup,
            create_backup=create_backup,
            backup_file=backup_file,
        )
    )


if __name__ == "__main__":
    create()



#
# Copyright (c) 2021 by Delphix. All rights reserved.
#

"""This module takes the conf file for DDP(s) and returns an authentication
   object
"""

import json
import ssl
from time import sleep,time

from delphixpy.v1_10_2 import exceptions
from delphixpy.v1_10_2 import job_context
from delphixpy.v1_10_2 import web
from delphixpy.v1_10_2.delphix_engine import DelphixEngine

from . import dlpx_exceptions
from . import dx_logging
from . import util
from .util import DXIConfigConstantsHelper

VERSION = "v.0.3.001"

class GetSession:
    """
    Class to read configuration and returns an Delphix session object
    """

    def __init__(self):
        self.server_session = None
        self.dlpx_ddps = {}
        self.jobs = {}

    def get_config(self, config_file_path=DXIConfigConstantsHelper().get_config()):
        """
        This method reads in the dxtools.conf file

        :param config_file_path: path to the configuration file.
        :type config_file_path: str
        :return: dict containing engine information
        """
        # First test to see that the file is there and we can open it
        try:
            with open(config_file_path) as config_file:
                config = json.loads(config_file.read())
        except IOError:
            raise dlpx_exceptions.DlpxException(
                f"\nERROR: Was unable to open {config_file_path}. Please "
                f"check the path and permissions, and try again.\n"
            )
        except (ValueError, TypeError, AttributeError) as err:
            raise dlpx_exceptions.DlpxException(
                f"\nERROR: Was unable to read {config_file_path} as json. "
                f"Please check if the file is in a json format and try "
                f"again.\n {err}"
            )
        for each in config.keys():
            temp_config = config[each]
            # HTTPS vs HTTP
            try:
                # temp_config["encrypted"].lower() == 'true'
                use_https = temp_config["use_https"]
            except Exception:
                use_https = "False"
            if use_https.lower() == "true":
                temp_config["use_https"] = True
            else:
                temp_config["use_https"] = False
            # Decryption Logic
            try:
                if temp_config["encrypted"].lower() == "true":
                    isEncrypted = True
                else:
                    isEncrypted = False
            except Exception:
                isEncrypted = False
            if isEncrypted:
                key = util.get_encryption_key(temp_config["ip_address"])
                temp_config["username"] = util.decrypt_data(
                    key, temp_config["username"]
                )
                temp_config["password"] = util.decrypt_data(
                    key, temp_config["password"]
                )
            self.dlpx_ddps[each] = temp_config

    def dlpx_session(
        self,
        f_engine_address,
        f_engine_username,
        f_engine_password=None,
        enable_https=True,
    ):
        """
        Method to setup the session with DDP
        :param f_engine_address: The DDP's address (IP/DNS Name)
        :type f_engine_address: str
        :param f_engine_username: Username to authenticate
        :type f_engine_username: str
        :param f_engine_password: User's password
        :type f_engine_password: str
        :param enable_https: Enable or disable HTTPS
        :type enable_https: bool
        :return: delphixpy.v1_10_2.delphix_engine.DelphixEngine object
        """
        f_engine_namespace = "DOMAIN"

        # Remove the next 3 lines if using in a production environment.
        # if not os.environ.list("PYTHONHTTPSVERIFY", "") and getattr(
        #     ssl, "_create_unverified_context", None
        # ):
        ssl._create_default_https_context = ssl._create_unverified_context
        try:
            self.server_session = DelphixEngine(
                f_engine_address,
                f_engine_username,
                f_engine_password,
                f_engine_namespace,
                enable_https,
            )
            self.server_wait(f_engine_address)
        except (
            exceptions.HttpError,
            exceptions.RequestError,
            exceptions.JobError,
        ) as err:
            raise dlpx_exceptions.DlpxException(
                f"ERROR: An error occurred while authenticating to "
                f"{f_engine_address}:\n {err}\n"
            )
        except (TimeoutError) as err:
            raise dlpx_exceptions.DlpxException(
                f"ERROR: Timeout while authenticating to "
                f"{f_engine_address}:\n {err}\n"
            )
        except (Exception) as err:
            raise dlpx_exceptions.DlpxException(
                f"ERROR: Error while authenticating to "
                f"{f_engine_address}:{err}"
            )

    def job_mode(self, single_thread=True):
        """
        This method tells the jobs to run sync or async, based on the
        single_thread variable
        :param single_thread: Execute application synchronously (True) or
                       async (False)
                       Default: True
        :type single_thread: Bool
        :return: contextlib._GeneratorContextManager
        """
        # Synchronously
        if single_thread:
            return job_context.sync(self.server_session)
        # Or asynchronously
        elif single_thread is False:
            return job_context.asyncly(self.server_session)

    def job_wait(self):
        """
        This job stops all work in the thread/process until jobs are
        completed.
        """
        # Grab all the jos on the server (the last 25, be default)
        all_jobs = web.job.get_all(self.server_session)
        # For each job in the list, check to see if it is running (not ended)
        for job_obj in all_jobs:
            if not (job_obj.job_state in ["CANCELED", "COMPLETED", "FAILED"]):
                dx_logging.print_debug(
                    f"\nDEBUG: Waiting for {job_obj.reference} "
                    f"(currently: {job_obj.job_state}) to finish running "
                    f"against the container.\n"
                )
                # If so, wait
                job_context.wait(self.server_session, job_obj.reference)

    def server_wait(self,f_engine_address):
        """
        This job waits for a successful connection to DDP.
        """
        timeout = time() + 60
        while True:
            try:
                web.system.get(self.server_session)
                break
            except (exceptions.HttpError) as err:
                err_msg = ""
                if hasattr(err,'status'):
                    if getattr(err,'status') == 401:
                        err_msg = f"Unable to login to Delphix Engine: " \
                                  f"{f_engine_address} with the provided credentials"
                        raise Exception(err_msg)
            except (exceptions.RequestError) as err:
                err_msg = f"Unable to login to Delphix Engine:" \
                          f"{f_engine_address} \n\t{err.args[0].details}\n\t" \
                          f"{err.args[0].action}"
                raise Exception(err_msg)
            except Exception as err:
                pass
            if time() > timeout:
                raise Exception(" Authentication request to Delphix Engine timedout [60s]")
            dx_logging.print_info("Waiting for response from Delphix Engine")
            sleep(5)


def dlpx_logout(dlpx_session):
    """
    Method to logout
    """
    try:
        web.delphix.delphix.logout(dlpx_session)
    except (exceptions.HttpError, exceptions.RequestError) as err:
        dx_logging.print_exception(f"Error logging out:\n{err}")



#
# Copyright (c) 2021 by Delphix. All rights reserved.
#

import click
from dxi._lib.util import boolean_based_system_exit
from dxi.environment.dxi_environment import DXIEnvironment
from dxi.environment.dxi_environment import EnvironmentConstants


@click.group(
    short_help="\b\nPerform Environment Operations.\n"
               "[add | refresh | enable | disable | "
               "updatehost \ndelete | list]"
)
def environment():
    """
    Linux/Unix & Windows Environment operations
    """
    pass


@environment.command()
@click.option(
    "--old-host", required=True, help="Old IP or HostName of the environment"
)
@click.option(
    "--new-host", required=True, help="New IP or HostName of the environment"
)
@click.option(
    "--engine",
    default=EnvironmentConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread\n"
    "[Default: {}]".format(EnvironmentConstants.SINGLE_THREAD),
    default=EnvironmentConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(EnvironmentConstants.POLL),
    default=EnvironmentConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=EnvironmentConstants.PARALLEL,
)
@click.option(
    "--config",
    help="Full path to the dxtools.conf file (including filename).",
    default=EnvironmentConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="Full path of the logs folder.",
    default=EnvironmentConstants.LOG_FILE_PATH,
)
def updatehost(
    engine,
    single_thread,
    parallel,
    poll,
    config,
    log_file_path,
    old_host,
    new_host,
):
    """
    Update an environment's IP address
    """
    env_obj = DXIEnvironment(
        engine=engine,
        parallel=parallel,
        poll=poll,
        config_file=config,
        log_file_path=log_file_path,
        single_thread=single_thread,
    )
    boolean_based_system_exit(
        env_obj.updatehost(old_host=old_host, new_host=new_host)
    )


@environment.command()
@click.option("--env-name", required=True, help="Name of the environment.")
@click.option(
    "--engine",
    default=EnvironmentConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread\n"
    "[Default: {}]".format(EnvironmentConstants.SINGLE_THREAD),
    default=EnvironmentConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(EnvironmentConstants.POLL),
    default=EnvironmentConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=EnvironmentConstants.PARALLEL,
)
@click.option(
    "--config",
    help="Full path to the dxtools.conf file (including filename).",
    default=EnvironmentConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="Full path of the logs folder.",
    default=EnvironmentConstants.LOG_FILE_PATH,
)
def delete(
    engine, single_thread, parallel, poll, config, log_file_path, env_name
):
    """
    Delete an environment.
    """
    env_obj = DXIEnvironment(
        engine=engine,
        parallel=parallel,
        poll=poll,
        config_file=config,
        log_file_path=log_file_path,
        single_thread=single_thread,
    )
    boolean_based_system_exit(env_obj.delete(env_name=env_name))


@environment.command()
@click.option("--env-name", required=True, help="Name of the environment.")
@click.option(
    "--engine",
    default=EnvironmentConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread\n"
    "[Default: {}]".format(EnvironmentConstants.SINGLE_THREAD),
    default=EnvironmentConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(EnvironmentConstants.POLL),
    default=EnvironmentConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=EnvironmentConstants.PARALLEL,
)
@click.option(
    "--config",
    help="Full path to the dxtools.conf file (including filename).",
    default=EnvironmentConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="Full path of the logs folder.",
    default=EnvironmentConstants.LOG_FILE_PATH,
)
def enable(
    engine, single_thread, parallel, poll, config, log_file_path, env_name
):
    """
    Enable an environment.
    """
    env_obj = DXIEnvironment(
        engine=engine,
        parallel=parallel,
        poll=poll,
        config_file=config,
        log_file_path=log_file_path,
        single_thread=single_thread,
    )
    boolean_based_system_exit(env_obj.enable(env_name=env_name))


@environment.command()
@click.option(
    "--engine",
    default=EnvironmentConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread\n"
    "[Default: {}]".format(EnvironmentConstants.SINGLE_THREAD),
    default=EnvironmentConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(EnvironmentConstants.POLL),
    default=EnvironmentConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=EnvironmentConstants.PARALLEL,
)
@click.option(
    "--config",
    help="Full path to the dxtools.conf file (including filename).",
    default=EnvironmentConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="Full path of the logs folder.",
    default=EnvironmentConstants.LOG_FILE_PATH,
)
@click.option("--env-name", required=True, help="Name of the environment.")
def disable(
    engine, single_thread, parallel, poll, config, log_file_path, env_name
):
    """
    Disable an environment.
    """
    env_obj = DXIEnvironment(
        engine=engine,
        parallel=parallel,
        poll=poll,
        config_file=config,
        log_file_path=log_file_path,
        single_thread=single_thread,
    )
    boolean_based_system_exit(env_obj.disable(env_name=env_name))


@environment.command()
@click.option("--env-name", required=True, help="Name of the environment.")
@click.option(
    "--engine",
    default=EnvironmentConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread\n"
    "[Default: {}]".format(EnvironmentConstants.SINGLE_THREAD),
    default=EnvironmentConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(EnvironmentConstants.POLL),
    default=EnvironmentConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=EnvironmentConstants.PARALLEL,
)
@click.option(
    "--config",
    help="Full path to the dxtools.conf file (including filename).",
    default=EnvironmentConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="Full path of the logs folder.",
    default=EnvironmentConstants.LOG_FILE_PATH,
)
def refresh(
    engine, single_thread, parallel, poll, config, log_file_path, env_name
):
    """
    Refresh an environment.
    """
    env_obj = DXIEnvironment(
        engine=engine,
        parallel=parallel,
        poll=poll,
        config_file=config,
        log_file_path=log_file_path,
        single_thread=single_thread,
    )
    boolean_based_system_exit(env_obj.refresh(env_name=env_name))


@environment.command()
@click.option(
    "--engine",
    default=EnvironmentConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread\n"
    "[Default: {}]".format(EnvironmentConstants.SINGLE_THREAD),
    default=EnvironmentConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(EnvironmentConstants.POLL),
    default=EnvironmentConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=EnvironmentConstants.PARALLEL,
)
@click.option(
    "--config",
    help="Full path to the dxtools.conf file (including filename).",
    default=EnvironmentConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="Full path of the logs folder.",
    default=EnvironmentConstants.LOG_FILE_PATH,
)
def list(engine, single_thread, parallel, poll, config, log_file_path):
    """
    List all environments on an engine
    """
    env_obj = DXIEnvironment(
        engine=engine,
        parallel=parallel,
        poll=poll,
        config_file=config,
        log_file_path=log_file_path,
        single_thread=single_thread,
    )
    boolean_based_system_exit(env_obj.list())


@environment.command()
@click.option("--env-name", required=True, help="Name of the environment.")
@click.option(
    "--env-type",
    required=True,
    default=EnvironmentConstants.TYPE,
    help="\b\nType of the environment. \n" "[ unix | windows ]",
)
@click.option(
    "--host-ip",
    required=True,
    help="IP address or Hostname of the environment.",
    default=EnvironmentConstants.HOSTIP,
)
@click.option(
    "--toolkit-dir",
    help="\b\nDirectory on the UNIX/LINUX environment\n"
    "to download Delphix Toolkit.\n"
    "[Required if adding a UNIX/LINUX environment]",
    default=None,
)
@click.option(
    "--os-user",
    required=True,
    help="Delphix OS user on the environment.",
    default=None,
)
@click.option(
    "--os-user-pwd",
    required=True,
    help="Delphix OS user password",
    default=None,
)
@click.option(
    "--connector-env-name",
    help="\b\nName of the environment on which \n"
    " Windows connector is installed and running.\n"
    "[Required if adding a Windows Source environment]",
    default=None,
)
@click.option(
    "--ase-db-user",
    help="\b\nDatabase username for ASE.\n"
    "[Required if adding an environment for ASE Database]",
    default=None,
)
@click.option(
    "--ase-db-user-pwd",
    help="\b\nDatabase user password for ASE.\n"
    "[Required if adding an environment for ASE Database]",
    default=None,
)
@click.option(
    "--engine",
    default=EnvironmentConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread\n"
    "[Default: {}]".format(EnvironmentConstants.SINGLE_THREAD),
    default=EnvironmentConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(EnvironmentConstants.POLL),
    default=EnvironmentConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=EnvironmentConstants.PARALLEL,
)
@click.option(
    "--config",
    help="Full path to the dxtools.conf file (including filename).",
    default=EnvironmentConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="Full path of the logs folder.",
    default=EnvironmentConstants.LOG_FILE_PATH,
)
def add(
    env_name,
    env_type,
    host_ip,
    toolkit_dir,
    connector_env_name,
    os_user,
    os_user_pwd,
    ase_db_user,
    ase_db_user_pwd,
    engine,
    single_thread,
    parallel,
    poll,
    config,
    log_file_path,
):
    """
    Add an environment.
    """
    env_obj = DXIEnvironment(
        engine=engine,
        parallel=parallel,
        poll=poll,
        config_file=config,
        log_file_path=log_file_path,
        single_thread=single_thread,
    )
    boolean_based_system_exit(
        env_obj.add(
            env_type=env_type,
            env_name=env_name,
            host_ip=host_ip,
            toolkit_dir=toolkit_dir,
            connector_env_name=connector_env_name,
            os_user=os_user,
            os_user_pwd=os_user_pwd,
            ase_db_user=ase_db_user,
            ase_db_user_pwd=ase_db_user_pwd,
        )
    )


if __name__ == "__main__":
    list()



#
# Copyright (c) 2021 by Delphix. All rights reserved.
#

"""Creates, updates, deletes, activates and lists branches

Examples:
  dxi_branch.py --list
  dxi_branch.py --create_branch jsbranch1 --container_name jscontainer \
  --template_name jstemplate1
  dxi_branch.py --activate_branch jsbranch1
  dxi_branch.py --delete_branch jsbranch1
  dxi_branch.py --create_branch tb4 --container_name dc1 \
    --timestamp "2021-02-07T04:34:48.952Z"
"""
import re

from delphixpy.v1_10_2 import exceptions
from delphixpy.v1_10_2.web import selfservice
from delphixpy.v1_10_2.web import vo
from dxi._lib import dlpx_exceptions
from dxi._lib import dx_logging
from dxi._lib import get_references
from dxi._lib import run_job
from dxi._lib.dxi_constants import BranchOps
from dxi._lib.run_async import run_async
from dxi._lib.util import DXIConfigConstantsHelper
from dxi.dxi_tool_base import DXIBase
from tabulate import tabulate


class DXIBranchConstants(object):
    """
    Define constants for SelfService Branch operations
    """

    SINGLE_THREAD = False
    POLL = 20
    CONFIG = DXIConfigConstantsHelper().get_config()
    LOG_FILE_PATH = DXIConfigConstantsHelper().get_logdir()
    ENGINE_ID = "default"
    PARALLEL = 5
    TYPE = None
    ACTION = None
    HOSTIP = None
    MODULE_NAME = __name__
    LIST_HEADER = [
        "Branch Name",
        "Data Layout",
        "Layout Type",
        "Branch Reference",
        "End Time",
    ]


class DXIBranch(DXIBase):
    """
    All Self Service Branch Operations
    """

    def __init__(
        self,
        engine=DXIBranchConstants.ENGINE_ID,
        log_file_path=DXIBranchConstants.LOG_FILE_PATH,
        config_file=DXIBranchConstants.CONFIG,
        poll=DXIBranchConstants.POLL,
        single_thread=DXIBranchConstants.SINGLE_THREAD,
        parallel=DXIBranchConstants.PARALLEL,
        action=DXIBranchConstants.ACTION,
        module_name=DXIBranchConstants.MODULE_NAME,
    ):
        """
        :param engine: Alt Identifier of Delphix engine in dxtools.conf.
        :type engine: `str`
        :param config: The path to the dxtools.conf file
        :type config: `str`
        :param log_file_path: The path to the logfile you want to use.
        :type log_file_path: `str`
        :param poll: The number of seconds to wait between job polls
        :type poll: `int`
        :param single_thread: Run as a single thread.
                              False if running multiple threads.
        :type single_thread: `bool`
        :param module_name: Name of the current module
        :type poll: `str`
        """
        super().__init__(
            parallel=parallel,
            poll=poll,
            config=config_file,
            log_file_path=log_file_path,
            single_thread=single_thread,
            module_name=module_name,
            engine=engine,
        )
        self.action = action
        self.branch_name = ""
        self.container_name = ""
        self.bookmark_name = ""
        self.template_name = ""
        self.timestamp = ""

    def list(self):
        """
        List all existing branches
        """
        self.action = BranchOps.LIST
        try:
            dx_logging.print_debug("Listing Branches")
            self._execute_operation(self._list_branch_helper)
            return True
        except Exception:
            return False

    def create(
        self,
        branch_name,
        container_name=None,
        template_name=None,
        bookmark_name=None,
        timestamp=None,
    ):
        """
        Creates a self service branch
        :param branch_name: Name of the branch to create
        :type password: `str`
        :param container_name: Name of the SS container
        :type password: `str`
        :param template_name: Name of the SS template
        :type password: `str`
        :param bookmark_name: Bookmark to create branch
        :type password: `str`
        :param timestamp: Timestamp to create branch
        :type password: `str`
        """
        self.action = BranchOps.CREATE
        self.branch_name = branch_name
        self.container_name = container_name
        self.bookmark_name = bookmark_name
        self.template_name = template_name
        self.timestamp = timestamp
        try:
            dx_logging.print_debug(f"Creating Branch {self.branch_name}")
            self._execute_operation(self._create_branch_helper)
            return True
        except Exception:
            return False

    def delete(self, branch_name):
        """
        Deletes a self service branch
        :param branch_name: Name of the branch to create
        :type password: `str`
        """
        self.action = BranchOps.DELETE
        self.branch_name = branch_name
        try:
            dx_logging.print_debug(f"Deleting Branch {self.branch_name}")
            self._execute_operation(self._delete_branch_helper)
            return True
        except Exception:
            return False

    def activate(self, branch_name, container_name=None):
        """
        Activates a self service branch
        :param branch_name: Name of the branch to create
        :type password: `str`
        """
        self.action = BranchOps.ACTIVATE
        self.branch_name = branch_name
        self.container_name = container_name
        try:
            dx_logging.print_debug(f"Activating Branch {self.branch_name}")
            self._execute_operation(self._activate_branch_helper)
            return True
        except Exception:
            return False

    @run_async
    def _create_branch_helper(self, engine_ref, dlpx_obj, single_thread):
        """
        Create a Self-Service Branch

        :param engine_ref: Dictionary of engines
        :type engine_ref: `dict`

        :param dlpx_obj: DDP session object
        :type dlpx_obj: `lib.GetSession.GetSession`

        :return: created template reference
        :rtype: `str`
        """
        dlpx_obj = self._initialize_session()
        self._setup_dlpx_session(dlpx_obj, engine_ref)
        with dlpx_obj.job_mode(self.single_thread):
            try:
                if not self.container_name:
                    raise dlpx_exceptions.DlpxObjectNotFound(
                        "A containername is required to create branch"
                    )
                ss_branch = vo.JSBranchCreateParameters()
                ss_branch.name = self.branch_name
                data_container_obj = get_references.find_obj_by_name(
                    dlpx_obj.server_session,
                    selfservice.container,
                    self.container_name,
                )
                ss_branch.data_container = data_container_obj.reference
                if self.timestamp:
                    ss_branch.timeline_point_parameters = (
                        vo.JSTimelinePointTimeInput()
                    )
                    ss_branch.timeline_point_parameters.time = get_references.convert_timestamp_toiso(  # noqa
                        dlpx_obj.server_session, self.timestamp
                    )
                    ss_branch.timeline_point_parameters.branch = (
                        data_container_obj.active_branch
                    )
                elif self.bookmark_name:
                    ss_branch.timeline_point_parameters = (
                        vo.JSTimelinePointBookmarkInput()
                    )
                    ss_branch.timeline_point_parameters.bookmark = get_references.find_obj_by_name(  # noqa
                        dlpx_obj.server_session,
                        selfservice.bookmark,
                        self.bookmark_name,
                    ).reference
                elif self.template_name or self.container_name:
                    if self.template_name:
                        source_layout_ref = get_references.find_obj_by_name(
                            dlpx_obj.server_session,
                            selfservice.template,
                            self.template_name,
                        ).reference
                    else:
                        source_layout_ref = get_references.find_obj_by_name(
                            dlpx_obj.server_session,
                            selfservice.container,
                            self.container_name,
                        ).reference
                    ss_branch.timeline_point_parameters = (
                        vo.JSTimelinePointLatestTimeInput()
                    )
                    ss_branch.timeline_point_parameters.source_data_layout = (
                        source_layout_ref
                    )

                selfservice.branch.create(dlpx_obj.server_session, ss_branch)
                self._add_last_job_to_track(dlpx_obj)
                run_job.track_running_jobs(
                    engine_ref,
                    dlpx_obj,
                    poll=self.poll,
                    failures=self.failures,
                )
                dx_logging.print_debug(
                    f"Self Service Branch {self.branch_name} has been created"
                )
            except (exceptions.RequestError, exceptions.HttpError) as err:
                dx_logging.print_exception(
                    f"There was an error while creating the branch: {err}"
                )
                # raise dlpx_exceptions.DlpxException(
                #     f"There was an error while creating the branch: {err}"
                # )
                self.failures[0] = True
            except (dlpx_exceptions.DlpxObjectNotFound) as err:
                dx_logging.print_exception(
                    f"ERROR Could not find a required object reference"
                    f" to create the branch: \nRoot cause: {err}"
                    f"\nPlease verify if the provided containername "
                    f"/ bookmarkname / timestamp exist on the engine "
                )
                # raise dlpx_exceptions.DlpxException(
                #     f"ERROR Could not find a required object reference"
                #     f" to create the branch: {err}"
                # )
                self.failures[0] = True
            except (Exception) as err:
                dx_logging.print_exception(
                    f"ERROR: Unable to create branch: {err}"
                )
                # raise dlpx_exceptions.DlpxException(
                #     f"ERROR Could not find a required object reference"
                #     f" to create the branch: {err}"
                # )
                self.failures[0] = True

    @run_async
    def _list_branch_helper(self, engine_ref, dlpx_obj, single_thread):
        """
        List all branches on a given engine

        :param engine_ref: Dictionary of engines
        :type engine_ref: `dict`

        :param dlpx_obj: DDP session object
        :type dlpx_obj: `lib.GetSession.GetSession`

        :return: created template reference
        :rtype: `str`
        """
        dlpx_obj = self._initialize_session()
        self._setup_dlpx_session(dlpx_obj, engine_ref)
        try:
            print_list = []
            js_data_layout = ""
            with dlpx_obj.job_mode(self.single_thread):
                branch_list = []
                ss_branches = selfservice.branch.get_all(
                    dlpx_obj.server_session
                )
                if ss_branches:
                    for ss_branch in ss_branches:
                        js_end_time = selfservice.operation.get(
                            dlpx_obj.server_session, ss_branch.first_operation
                        ).end_time
                        js_obj_type = "CONTAINER"
                        if re.search("TEMPLATE", ss_branch.data_layout):
                            js_data_layout = get_references.find_obj_name(
                                dlpx_obj.server_session,
                                selfservice.template,
                                ss_branch.data_layout,
                            )
                            js_obj_type = "TEMPLATE"
                        elif re.search("CONTAINER", ss_branch.data_layout):
                            js_data_layout = get_references.find_obj_name(
                                dlpx_obj.server_session,
                                selfservice.container,
                                ss_branch.data_layout,
                            )
                        print_list.append(
                            [
                                ss_branch._name[0],
                                js_data_layout,
                                js_obj_type,
                                ss_branch.reference,
                                js_end_time,
                            ]
                        )
                        branch_list.append(
                            dict(
                                zip(
                                    DXIBranchConstants.LIST_HEADER,
                                    [
                                        ss_branch._name[0],
                                        js_data_layout,
                                        js_obj_type,
                                        ss_branch.reference,
                                        js_end_time,
                                    ],
                                )
                            )
                        )
                else:
                    dx_logging.print_info(f"No branches found on engine.")
                self._add_last_job_to_track(dlpx_obj)
                run_job.track_running_jobs(
                    engine_ref,
                    dlpx_obj,
                    poll=self.poll,
                    failures=self.failures,
                )
                print("\n")
                print(
                    tabulate(
                        print_list,
                        headers=DXIBranchConstants.LIST_HEADER,
                        tablefmt="grid",
                    )
                )
                return branch_list
        except dlpx_exceptions.DlpxException as err:
            dx_logging.print_exception(
                f"ERROR: Could not list self service branches:{err} "
            )
            # raise (f"ERROR: Could not list self service branches:{err} ")
            self.failures[0] = True
        except Exception as err:
            dx_logging.print_exception(
                f"ERROR: Could not list self service branches:{err} "
            )
            # raise (f"ERROR: Could not list self service branches:{err} ")
            self.failures[0] = True

    @run_async
    def _activate_branch_helper(self, engine_ref, dlpx_obj, single_thread):
        """
        Activates a branch

        :param engine_ref: Dictionary of engines
        :type engine_ref: `dict`

        :param dlpx_obj: DDP session object
        :type dlpx_obj: `lib.GetSession.GetSession`

        :return: List of all templtes on a given engine
        :rtype: `list`
        """
        dlpx_obj = self._initialize_session()
        self._setup_dlpx_session(dlpx_obj, engine_ref)
        try:
            with dlpx_obj.job_mode(self.single_thread):
                data_container_obj = get_references.find_obj_by_name(
                    dlpx_obj.server_session,
                    selfservice.container,
                    self.container_name,
                )
                branch_obj = get_references.find_branch_by_name_and_container_ref(  # noqa
                    dlpx_obj.server_session,
                    self.branch_name,
                    data_container_obj.reference,
                )
                # branch_obj = get_references.find_obj_by_name(
                #     dlpx_obj.server_session,
                #     selfservice.branch,
                #     self.branch_name,
                # )
                selfservice.branch.activate(
                    dlpx_obj.server_session, branch_obj.reference
                )
                self._add_last_job_to_track(dlpx_obj)
                run_job.track_running_jobs(
                    engine_ref,
                    dlpx_obj,
                    poll=self.poll,
                    failures=self.failures,
                )
                dx_logging.print_info(
                    f"The branch {self.branch_name} activated"
                )
        except (
            exceptions.RequestError,
            dlpx_exceptions.DlpxObjectNotFound,
            Exception,
        ) as err:
            dx_logging.print_exception(
                f"ERROR: An error occurred activating "
                f"the {self.branch_name}:{err}"
            )
            # raise dlpx_exceptions.DlpxException(
            #     f"ERROR: An error occurred activating "
            #     f"the {self.branch_name}:{err}"
            # )
            self.failures[0] = True

    @run_async
    def _delete_branch_helper(self, engine_ref, dlpx_obj, single_thread):
        """
        Deletes a branch

        :param engine_ref: Dictionary of engines
        :type engine_ref: `dict`

        :param dlpx_obj: DDP session object
        :type dlpx_obj: `lib.GetSession.GetSession`

        :return: List of all templtes on a given engine
        :rtype: `list`
        """
        dlpx_obj = self._initialize_session()
        self._setup_dlpx_session(dlpx_obj, engine_ref)
        try:
            with dlpx_obj.job_mode(self.single_thread):
                branch_obj = get_references.find_obj_by_name(
                    dlpx_obj.server_session,
                    selfservice.branch,
                    self.branch_name,
                )
                selfservice.branch.delete(
                    dlpx_obj.server_session, branch_obj.reference
                )
                self._add_last_job_to_track(dlpx_obj)
                run_job.track_running_jobs(
                    engine_ref,
                    dlpx_obj,
                    poll=self.poll,
                    failures=self.failures,
                )
        except (
            dlpx_exceptions.DlpxException,
            exceptions.HttpError,
            exceptions.RequestError,
        ) as err:
            dx_logging.print_exception(
                f"The branch could not be deleted: {err}"
            )
            # raise dlpx_exceptions.DlpxException(
            #     f"ERROR: The branch was not deleted : {err}"
            # )
            self.failures[0] = True
        except Exception as err:
            dx_logging.print_exception(
                f"The branch could not be deleted: {err}"
            )
            # raise dlpx_exceptions.DlpxException(
            #     f"ERROR: The branch was not deleted : {err}"
            # )
            self.failures[0] = True



#
# Copyright (c) 2021 by Delphix. All rights reserved.
#
from dxi._lib.util import DXIConfigConstantsHelper
from dxi.database._internal_lib._dxi_db import DXIDb
from dxi.database._internal_lib._dxi_ingest import DXIDsource
from dxi.database._internal_lib._dxi_provision import DXIVdb


class DXIDatabaseConstants(object):
    """
    Class of common constants used by Provision dSource
    """

    SINGLE_THREAD = False
    POLL = 10
    CONFIG = DXIConfigConstantsHelper().get_config()
    LOG_FILE_PATH = DXIConfigConstantsHelper().get_logdir()
    ENGINE_ID = "default"
    PARALLEL = 5
    ACTION = None
    MODULE_NAME = __name__
    VDB_LIST_HEADER = []
    FORCE = False
    TYPE = "vdb"
    ORACLE_DEFAULT_PORT = "1521"


class DXIDatabase(object):
    """
    A class which use to perform all data set related operations:
    Refresh, Ingest, Provision, Start, Stop, Disable, Enable
    """

    def __init__(self, *args, **kwargs):
        """
        Args:
            engine: Name of the database.
            single_thread: Name of the group, if specifying one.
            config: The path to the dxtools.conf file
            log_file_path: The path to the logfile you want to use.
            poll: The number of seconds to wait between job polls .
        """
        kwargs.setdefault("engine", DXIDatabaseConstants.ENGINE_ID)
        kwargs.setdefault("log_file_path", DXIDatabaseConstants.LOG_FILE_PATH)
        kwargs.setdefault("config", DXIDatabaseConstants.CONFIG)
        kwargs.setdefault("poll", DXIDatabaseConstants.POLL)
        kwargs.setdefault("single_thread", DXIDatabaseConstants.SINGLE_THREAD)
        kwargs.setdefault("parallel", DXIDatabaseConstants.PARALLEL)
        kwargs.setdefault("module_name", DXIDatabaseConstants.MODULE_NAME)
        self._ingest = DXIDsource(**kwargs)
        self._provision = DXIVdb(**kwargs)
        self._db_common = DXIDb(**kwargs)

    # all methods related to common db operations
    def list(self):
        """
        List all data sets on a Delphix Engine.
        Raises:
            General Exception
        Returns:
            boolean: Indicates success or failure of the operation.
        """
        return self._db_common.list()

    def start(self, db_name, group=None):
        """
        Start a virtual dataset.

        Args:
            db_name: Name of the database.
            group: Name of the group, if specifying one.
        Raises:
            General Exception
        Returns:
            boolean: Indicates success or failure of the operation.
        """
        return self._db_common.start(db_name, group)

    def stop(self, db_name, group=None):
        """
        Stop a vitual dataset.
        Args:
            db_name: Name of the database.
            group: Name of the group, if specifying one.
        Raises:
            General Exception
        Returns:
            boolean: Indicates success or failure of the operation.
        """
        return self._db_common.stop(db_name, group)

    def enable(self, db_name, group=None):
        """
        Enable a virtual database.
        Args:
            db_name: Name of the database.
            group: Name of the group, if specifying one.
        Raises:
            General Exception
        Returns:
            boolean: Indicates success or failure of the operation.
        """
        return self._db_common.enable(db_name, group)

    def disable(self, db_name, group=None, force=False):
        """
        Disable a virtual database.
        Args:
            db_name: Name of the database.
            group: Name of the group, if specifying one.
        Raises:
            General Exception
        Returns:
            boolean: Indicates success or failure of the operation.
        """
        return self._db_common.disable(db_name, group, force)

    def refresh(
        self,
        db_name,
        timestamp_type="SNAPSHOT",
        timestamp="LATEST",
        timeflow="None",
    ):
        """
        Refresh a Delphix VDB or vFile.
        Args:
            db_name: Name of the database to refresh.
            timestamp_type: Type of timestamp to use for refresh.
                            SNAPSHOT | TIME
            timestamp: Timestamp reference for refresh.
            timeflow: Name of the timeflow for the refresh.
        Raises:
            General Exception
        Returns:
            boolean: Indicates success or failure of the operation.
        """
        return self._provision.refresh(
            db_name, timestamp_type, timestamp, timeflow
        )

    def delete(self, db_name, db_type="vdb", force=False):
        """
        Delete a virtual or physical dataset
        Args:
            db_name: Name of the database/databases.
            db_type: Type of database. vdb/dsource
            force: Whether to force delete the datasets.
        Raises:
            General Exception
        Returns:
            boolean: Indicates success or failure of the operation.
        """
        return self._provision.delete(db_name, db_type, force)

    def rewind(
        self,
        db_name,
        timestamp_type="SNAPSHOT",
        timestamp="LATEST",
        db_type=None,
    ):
        """
        Rewind a virtual dataset.
        Args:
            db_name: Name of the database/databases.
            timestamp_type: Type of timestamp to use for refresh.
                            SNAPSHOT | TIME
            timestamp: Timestamp reference for refresh.
            timeflow: Name of the timeflow for the refresh.
        Raises:
            General Exception
        Returns:
            boolean: Indicates success or failure of the operation.
        """
        return self._provision.rewind(
            db_name, timestamp_type, timestamp, db_type
        )

    def provision_postgres(
        self,
        group,
        source_name,
        db_name,
        env_name,
        prerefresh=False,
        postrefresh=False,
        prerollback=False,
        postrollback=False,
        configure_clone=False,
        port_num=5432,
        timestamp_type="SNAPSHOT",
        timestamp="LATEST",
        mntpoint="/mnt/ingest",
        env_inst=None,
    ):
        """
        Provision a Postgres VDB.
        Args:
            db_name           Name for the VDB/vFile.
            source_name       The source database.
            env_name         The name of the target environment in Delphix.
            port_num          The port number of the database instance.
            env_inst          Full path to database binary on target.
            group             The group into which Delphix will place the VDB.
            timestamp_type    The type of timestamp [TIME | SNAPSHOT]
            timestamp         The point in time on the source db or vfile
            mntpoint          Mount folder that Delphix can use on the target
            prerefresh        Hook name to run before a refresh only
            postrefresh       Hook name to run after a refresh only
            prerollback       Hook name to run before a rewind
            postrollback      Hook name to run after a rewind
            configure_clone   Hook name to run after provision/refresh.
        Raises:
            General Exception
        Returns:
            boolean: Indicates success or failure of the operation.
        """
        return self._provision.provision_postgres(
            group,
            source_name,
            db_name,
            env_name,
            prerefresh,
            postrefresh,
            prerollback,
            postrollback,
            configure_clone,
            port_num,
            timestamp_type,
            timestamp,
            mntpoint,
            env_inst,
        )

    def provision_ase(
        self,
        group,
        source_name,
        db_name,
        env_name,
        prerefresh=False,
        postrefresh=False,
        prerollback=False,
        postrollback=False,
        configure_clone=False,
        port_num=5432,
        timestamp_type="SNAPSHOT",
        timestamp="LATEST",
        mntpoint="/mnt/ingest",
        env_inst=None,
        no_truncate_log=False,
    ):
        """
        Provision a ASE VDB.
        Args:
            db_name           Name for the VDB/vFile.
            source_name       The source database.
            env_name         The name of the target environment in Delphix.
            port_num          The port number of the database instance.
            env_inst          Full path to database binary on target.
            group             The group into which Delphix will place the VDB.
            timestamp_type    The type of timestamp [TIME | SNAPSHOT]
            timestamp         The point in time on the source db or vfile
            mntpoint          Mount folder that Delphix can use on the target
            prerefresh        Hook name to run before a refresh only
            postrefresh       Hook name to run after a refresh only
            prerollback       Hook name to run before a rewind
            postrollback      Hook name to run after a rewind
            configure_clone   Hook name to run after provision/refresh.
            no_truncate_log   Set the trunc log on chkpt database option
        Raises:
            General Exception
        Returns:
            boolean: Indicates success or failure of the operation.
        """
        return self._provision.provision_ase(
            group,
            source_name,
            db_name,
            env_name,
            prerefresh,
            postrefresh,
            prerollback,
            postrollback,
            configure_clone,
            port_num,
            timestamp_type,
            timestamp,
            mntpoint,
            env_inst,
            no_truncate_log,
        )

    def provision_mssql(
        self,
        group,
        source_name,
        db_name,
        env_name,
        prerefresh=False,
        postrefresh=False,
        prerollback=False,
        postrollback=False,
        configure_clone=False,
        port_num=5432,
        timestamp_type="SNAPSHOT",
        timestamp="LATEST",
        mntpoint="/mnt/ingest",
        env_inst=None,
    ):
        """
        Provision a MSSQL VDB.
        Args:
            db_name           Name for the VDB/vFile.
            source_name       The source database.
            env_name         The name of the target environment in Delphix.
            port_num          The port number of the database instance.
            env_inst          Full path to database binary on target.
            group             The group into which Delphix will place the VDB.
            timestamp_type    The type of timestamp [TIME | SNAPSHOT]
            timestamp         The point in time on the source db or vfile
            mntpoint          Mount folder that Delphix can use on the target
            prerefresh        Hook name to run before a refresh only
            postrefresh       Hook name to run after a refresh only
            prerollback       Hook name to run before a rewind
            postrollback      Hook name to run after a rewind
            configure_clone   Hook name to run after provision/refresh.
        Raises:
            General Exception
        Returns:
            boolean: Indicates success or failure of the operation.
        """
        return self._provision.provision_mssql(
            group,
            source_name,
            db_name,
            env_name,
            prerefresh,
            postrefresh,
            prerollback,
            postrollback,
            configure_clone,
            port_num,
            timestamp_type,
            timestamp,
            mntpoint,
            env_inst,
        )

    def provision_vfiles(
        self,
        group,
        source_name,
        db_name,
        env_name,
        prerefresh=False,
        postrefresh=False,
        prerollback=False,
        postrollback=False,
        configure_clone=False,
        port_num=5432,
        timestamp_type="SNAPSHOT",
        timestamp="LATEST",
        mntpoint="/mnt/ingest",
        env_inst=None,
    ):
        """
        Create a vfiles VDB
        Args:
            db_name           Name for the VDB/vFile.
            source_name       The source database.
            env_name         The name of the target environment in Delphix.
            port_num          The port number of the database instance.
            env_inst          Full path to database binary on target.
            group             The group into which Delphix will place the VDB.
            timestamp_type    The type of timestamp [TIME | SNAPSHOT]
            timestamp         The point in time on the source db or vfile
            mntpoint          Mount folder that Delphix can use on the target
            prerefresh        Hook name to run before a refresh only
            postrefresh       Hook name to run after a refresh only
            prerollback       Hook name to run before a rewind
            postrollback      Hook name to run after a rewind
            configure_clone   Hook name to run after provision/refresh.
        Raises:
            General Exception
        Returns:
            boolean: Indicates success or failure of the operation.

        """
        return self._provision.provision_vfiles(
            group,
            source_name,
            db_name,
            env_name,
            prerefresh,
            postrefresh,
            prerollback,
            postrollback,
            configure_clone,
            port_num,
            timestamp_type,
            timestamp,
            mntpoint,
            env_inst,
        )

    def provision_oracle_si(
        self,
        group,
        source_name,
        db_name,
        env_name,
        prerefresh=False,
        postrefresh=False,
        prerollback=False,
        postrollback=False,
        configure_clone=False,
        port_num=5432,
        timestamp_type="SNAPSHOT",
        timestamp="LATEST",
        mntpoint="/mnt/ingest",
        env_inst=None,
    ):
        """
        Create an Oracle SI VDB
        Args:
            db_name           Name for the VDB/vFile.
            source_name       The source database.
            env_name         The name of the target environment in Delphix.
            port_num          The port number of the database instance.
            env_inst          Full path to database binary on target.
            group             The group into which Delphix will place the VDB.
            timestamp_type    The type of timestamp [TIME | SNAPSHOT]
            timestamp         The point in time on the source db or vfile
            mntpoint          Mount folder that Delphix can use on the target
            prerefresh        Hook name to run before a refresh only
            postrefresh       Hook name to run after a refresh only
            prerollback       Hook name to run before a rewind
            postrollback      Hook name to run after a rewind
            configure_clone   Hook name to run after provision/refresh.
        Raises:
            General Exception
        Returns:
            boolean: Indicates success or failure of the operation.

        """
        return self._provision.provision_oracle_si(
            group,
            source_name,
            db_name,
            env_name,
            prerefresh,
            postrefresh,
            prerollback,
            postrollback,
            configure_clone,
            port_num,
            timestamp_type,
            timestamp,
            mntpoint,
            env_inst,
        )

    def provision_oracle_mt(
        self,
        group,
        source_name,
        db_name,
        env_name,
        prerefresh=False,
        postrefresh=False,
        prerollback=False,
        postrollback=False,
        configure_clone=False,
        port_num=5432,
        timestamp_type="SNAPSHOT",
        timestamp="LATEST",
        mntpoint="/mnt/ingest",
        env_inst=None,
    ):
        """
        Create an Oracle Multi Tenant VDB
        Args:
            db_name           Name for the VDB/vFile.
            source_name       The source database.
            env_name         The name of the target environment in Delphix.
            port_num          The port number of the database instance.
            env_inst          Full path to database binary on target.
            group             The group into which Delphix will place the VDB.
            timestamp_type    The type of timestamp [TIME | SNAPSHOT]
            timestamp         The point in time on the source db or vfile
            mntpoint          Mount folder that Delphix can use on the target
            prerefresh        Hook name to run before a refresh only
            postrefresh       Hook name to run after a refresh only
            prerollback       Hook name to run before a rewind
            postrollback      Hook name to run after a rewind
            configure_clone   Hook name to run after provision/refresh.
        Raises:
            General Exception
        Returns:
            boolean: Indicates success or failure of the operation.

        """
        return self._provision.provision_oracle_mt(
            group,
            source_name,
            db_name,
            env_name,
            prerefresh,
            postrefresh,
            prerollback,
            postrollback,
            configure_clone,
            port_num,
            timestamp_type,
            timestamp,
            mntpoint,
            env_inst,
        )

    def provision_oracle_rac(
        self,
        group,
        source_name,
        db_name,
        env_name,
        nodes,
        prerefresh=False,
        postrefresh=False,
        prerollback=False,
        postrollback=False,
        configure_clone=False,
        port_num=1521,
        timestamp_type="SNAPSHOT",
        timestamp="LATEST",
        mntpoint="/mnt/provision",
        env_inst=None,
    ):
        """
        Create an Oracle RAC VDB
        Args:
            group             The group into which Delphix will place the VDB
            source-name       The source database.
            db-name           The name you want to give the database.
            env-name          The name of the Target environment in Delphix
            nodes             Colon separated list of nodes in RAC.
            port-num          The port number of the database instance
            prerefresh        Hook name to run before a refresh only
            postrefresh       Hook name to run after a refresh only
            prerollback       Hook name to run before a rewind
            postrollback      Hook name to run after a rewind
            configure_clone   Hook name to run after provision/refresh.
            env-inst          The identifier of the instance in Delphix.
            timestamp-type    The type of timestamp you are specifying. TIME or
            timestamp        The Delphix semantic for the point in time from
            mntpoint          The identifier of the instance in Delphix.
        Raises:
            General Exception
        Returns:
            boolean: Indicates success or failure of the operation.

        """
        return self._provision.provision_oracle_rac(
            group,
            source_name,
            db_name,
            env_name,
            nodes,
            prerefresh,
            postrefresh,
            prerollback,
            postrollback,
            configure_clone,
            port_num,
            timestamp_type,
            timestamp,
            mntpoint,
            env_inst,
        )

    # dSource methods
    def ingest_mssql(
        self,
        source_name,
        db_password,
        db_user,
        group,
        source_env,
        stage_env,
        stage_instance,
        logsync=True,
        validated_sync_mode="TRANSACTION_LOG",
        initial_load_type=None,
        delphix_managed=True,
        backup_path=None,
        backup_user_pwd=None,
        backup_user=None,
    ):
        """
        Link an MSSQL dSource
        Args:
            source_name           Name of the dSource to create.
            db_user               Dsource database username.
            db_passwd             Password for db_user.
            group                 Group name for this dSource.
            logsync               Enable or disable logsync
            source_env            Name of the source Db environment.
            stage_instance        Name of the staging instance for ingestion.
            stage_env             Name of the staging environment.
            backup_user_pwd       Password of the shared backup path.
            backup_user           User of the shared backup path
            validated_sync_mode   Validated Sync Mode to create the dsource.
            delphix_managed       If set, Delphix will manage the backups.
            initial_load_type     Initial load to create the MSSQL dsource.
            backup_path           Full path to the MS SQLServer backups
        Raises:
            General Exception
        Returns:
            boolean: Indicates success or failure of the operation.

        """
        return self._ingest.ingest_mssql(
            source_name,
            db_password,
            db_user,
            group,
            logsync,
            validated_sync_mode,
            initial_load_type,
            delphix_managed,
            stage_env,
            stage_instance,
            backup_path,
            backup_user_pwd,
            backup_user,
            source_env,
        )

    def ingest_sybase(
        self,
        source_name,
        db_user,
        db_password,
        group,
        source_env,
        stage_env,
        stage_instance,
        backup_path,
        backup_files=None,
        logsync=False,
        create_backup=None,
    ):
        """
        Link an Sybase dSource
        Args:
            source_name     Name of the dSource to create.
            db_user         Source database username.
            db_password     Password for db_user.
            group           Group name for this dSource
            env_name        Staging environment name
            source_env      Source environment name
            stage_instance  The sybase instance on the staging environment.
            backup_path     Path to the Sybase backups.
            logsync         Enable or disable logsync.
            backup_file     Backup file list (add space between multiple files)
            create_backup   The parameters to use as input to sync a SAP ASE
        Raises:
            General Exception
        Returns:
            boolean: Indicates success or failure of the operation.

        """
        return self._ingest.ingest_sybase(
            source_name,
            db_user,
            db_password,
            group,
            source_env,
            stage_env,
            stage_instance,
            backup_path,
            backup_files,
            logsync,
            create_backup,
        )

    def ingest_oracle(
        self,
        source_name,
        db_password,
        db_user,
        group,
        env_inst,
        logsync=True,
        env_name=None,
        ip_addr=None,
        port_num=None,
        rman_channels=2,
        files_per_set=5,
        num_connections=5,
    ):
        """
        Link an Oracle dSource
        Args:
            source_name     Name of the dSource to create.
            db_user         Dsource db username.
            db_password     Password for dsource db_user.
            group           Group name for the dsource.
            env_name        Name of the host environment for the dsource.
            env_inst        Path to oracle installation on the env.
            port_num        Port number for the oracle listener.
            logsync         Enable or disable logsync.
            ip_addr         IP Address of the dsource.
            rman_channels    Sets number of Oracle RMAN Channels.
            files_per_set    Sets number of files per set for Oracle.
            num_connections  Sets number of connections for Oracle RMAN.
        Raises:
            General Exception
        Returns:
            boolean: Indicates success or failure of the operation.

        """
        return self._ingest.ingest_oracle(
            source_name,
            db_password,
            db_user,
            group,
            logsync,
            env_name,
            env_inst,
            ip_addr,
            port_num,
            rman_channels,
            files_per_set,
            num_connections,
        )



#
# Copyright (c) 2021 by Delphix. All rights reserved.
#

import copy
import time
from abc import ABC
from os.path import basename

from dxi._lib import dlpx_exceptions
from dxi._lib import dx_logging
from dxi._lib import get_session
from dxi._lib import run_job
from dxi._lib import util


class DXIBase(ABC):
    """
    This class cannot be directly instantiated as it is abstract class.
    It also contains common properties like poll, single_thread etc.
    """

    def __init__(
        self,
        poll,
        config,
        log_file_path,
        single_thread,
        module_name,
        engine=None,
        parallel=5,
    ):
        self.parallel = parallel
        self.poll = poll
        self.single_thread = single_thread
        self.module_name = module_name
        self.config = util.find_config_path(config)
        self.engine = engine
        self.log_file_path = util.find_log_path(log_file_path, module_name)
        self.dx_session_obj = self._initialize_session()
        self.time_start = time.time()
        self.failures = [False]
        #self.session_mgr = self._initialize_session() # TODO
        dx_logging.logging_est(self.log_file_path)

    def _initialize_session(self):
        dx_session_obj = get_session.GetSession()
        dx_session_obj.get_config(config_file_path=self.config)
        return dx_session_obj

    def _remove_session(self, session):
        get_session.dlpx_logout(session)

    def display_choices(self, obj):
        dx_logging.print_debug("-" * 79)
        dx_logging.print_debug("Provided parameters for execution")
        dx_logging.print_debug("-" * 79)
        obj2 = copy.deepcopy(obj)
        attrs = vars(obj2)
        pw_keys = [
            key
            for key, value in attrs.items()
            if ("pass" in key.lower() or "user" in key.lower())
        ]
        for pw_key in pw_keys:
            attrs[pw_key] = "*" * 10
        dx_logging.print_debug(
            "\n ".join("%s: %s" % item for item in attrs.items())
        )
        dx_logging.print_debug("-" * 79)

    def _execute_operation(self, run_async_function_ref, helper_ref=None):
        """
        :param run_async_function_ref: A function reference
        :type run_async_function_ref: `callable`
        """
        try:
            for each in run_job.run_job_mt(
                run_async_function_ref,
                self.dx_session_obj,     # engine data
                self.engine,
                self.single_thread,
                helper_ref=helper_ref
            ):
                each.join()

            if self.failures[0] is True:
                raise
        except Exception as err:
            dx_logging.print_exception(err)
            raise
        elapsed_minutes = run_job.time_elapsed(self.time_start)
        dx_logging.print_info(
            f"dxi operation took {elapsed_minutes} minutes to complete"
        )

    @staticmethod
    def _setup_dlpx_session(dlpx_obj, engine_ref):
        """
        :param dlpx_obj: DDP session object
        :type dlpx_obj: `lib.GetSession.GetSession`
        :param engine_ref: Dictionary of engines
        :type engine_ref: `dict`
        """
        try:
            # Setup the connection to the Delphix DDP
            dx_logging.print_debug("Setting up the dlpx session")
            dlpx_obj.dlpx_session(
                engine_ref["ip_address"],
                engine_ref["username"],
                engine_ref["password"],
                engine_ref["use_https"],
            )
            dx_logging.print_debug("Connection established")
        except (dlpx_exceptions.DlpxException, Exception) as err:
            dx_logging.print_exception(
                f"ERROR: {basename(__file__)} encountered an error "
                f"authenticating "
                f'to {engine_ref["hostname"]}\n{err}'
            )
            raise dlpx_exceptions.DlpxException(
                f"ERROR: {basename(__file__)} encountered an error "
                f"authenticating "
                f'to {engine_ref["hostname"]}\n{err}'
            )

    @staticmethod
    def _add_last_job_to_track(dlpx_obj):
        if dlpx_obj.server_session.last_job:
            if dlpx_obj.server_session.address in dlpx_obj.jobs:
                dlpx_obj.jobs[dlpx_obj.server_session.address].append(
                    dlpx_obj.server_session.last_job
                )
            else:
                dlpx_obj.jobs[dlpx_obj.server_session.address] = [
                    dlpx_obj.server_session.last_job
                ]



#
# Copyright (c) 2021 by Delphix. All rights reserved.
#

from os.path import basename

from delphixpy.v1_10_2 import exceptions
from delphixpy.v1_10_2.web import source
from delphixpy.v1_10_2.web.capacity import consumer
from delphixpy.v1_10_2.web.vo import SourceDisableParameters
from dxi._lib import dlpx_exceptions as dxe
from dxi._lib import dx_logging as log
from dxi._lib import get_references as ref
from dxi._lib import run_job
from dxi._lib.dxi_constants import VirtualOps
from dxi._lib.run_async import run_async
from dxi._lib.util import DXIConfigConstantsHelper
from dxi.dxi_tool_base import DXIBase


class DXIDbConstants(object):
    """
    Define constants for DXIVDB class
    """

    SINGLE_THREAD = False
    POLL = 20
    CONFIG = DXIConfigConstantsHelper().get_config()
    LOG_FILE_PATH = DXIConfigConstantsHelper().get_logdir()
    ENGINE_ID = "default"
    TIME_STAMP_TYPE = "SNAPSHOT"
    TIME_STAMP = "LATEST"
    TIME_FLOW = None
    PARALLEL = 5
    FORCE = False
    TYPE = "vdb"
    DB_TYPE = None


class DXIDb(DXIBase):
    """
    Implement operations for vdb datasets
    """

    def __init__(
        self,
        engine=DXIDbConstants.ENGINE_ID,
        single_thread=DXIDbConstants.SINGLE_THREAD,
        poll=DXIDbConstants.POLL,
        config=DXIDbConstants.CONFIG,
        log_file_path=DXIDbConstants.LOG_FILE_PATH,
        parallel=DXIDbConstants.PARALLEL,
        module_name=__name__,
    ):
        """
          :param engine: An Identifier of Delphix engine in dxtools.conf.
          :type engine: `str`
          :param single_thread: Run as a single thread.
                 False if running multiple threads.
          :type single_thread: `bool`
          :param poll: The number of seconds to wait between job polls
          :type poll: `int`
          :param config: The path to the dxtools.conf file
          :type config: `str`
          :param log_file_path: The path to the logfile you want to use.
          :type log_file_path: `str`
        """
        super().__init__(
            poll=poll,
            config=config,
            log_file_path=log_file_path,
            single_thread=single_thread,
            engine=engine,
            module_name=module_name,
            parallel=parallel,
        )
        self.vdb = ""
        self.time_stamp_type = ""
        self.time_stamp = ""
        self.time_flow = ""
        self.name = ""
        self.force = ""
        self.type = ""
        self.db_type = ""
        self.source_db = ""
        self.group = ""
        self.db_type = ""
        self.env_name = ""
        self.mntpoint = ""
        self.timestamp = ""
        self.timestamp_type = ""
        self.pre_refresh = ""
        self.post_refresh = ""
        self.pre_rollback = ""
        self.post_rollback = ""
        self.configure_clone = ""
        self.envinst = ""
        self.action = ""
        self.port_num = ""

    def list(self):
        """
        List datasets on an engine
        """
        self.action = VirtualOps.LIST
        try:
            self._execute_operation(self._db_operation_helper)
            log.print_debug("End of Execution")
            return True
        except Exception as err:
            log.print_exception(
                f"An Error was encountered while "
                f"listing the datasets: {repr(err)}"
            )
            return False

    def start(self, name, group=None):
        """
        Start a Virtual dataset by name
        """
        self.action = VirtualOps.START
        self.name = name
        self.group = group
        try:
            self._execute_operation(self._db_operation_helper)
            log.print_debug("End of Execution")
            return True
        except Exception as err:
            log.print_exception(
                f"An Error was encountered while "
                f"starting the virtual dataset: {repr(err)}"
            )
            return False

    def stop(self, name, group=None):
        """
        Stop a Virtual dataset by name
        """
        self.action = VirtualOps.STOP
        self.name = name
        self.group = group
        try:
            self._execute_operation(self._db_operation_helper)
            log.print_debug("End of Execution")
            return True
        except Exception as err:
            log.print_exception(
                f"An Error was encountered while "
                f"stoping the virtual dataset: {repr(err)}"
            )
            return False

    def enable(self, name, group=None):
        """
        Enable a Virtual dataset by name
        """
        self.action = VirtualOps.ENABLE
        self.name = name
        self.group = group
        try:
            self._execute_operation(self._db_operation_helper)
            log.print_debug("End of Execution")
            return True
        except Exception as err:
            log.print_exception(
                f"An Error was encountered while "
                f"enabling the virtual dataset: {repr(err)}"
            )
            return False

    def disable(self, name, group=None, force=False):
        """
        Disable a Virtual dataset by name
        """
        self.action = VirtualOps.DISABLE
        self.name = name
        self.group = group
        self.force = force
        try:
            self._execute_operation(self._db_operation_helper)
            log.print_debug("End of Execution")
            return True
        except Exception as err:
            log.print_exception(
                f"An Error was encountered while "
                f"disabling the virtual dataset: {repr(err)}"
            )
            return False

    def _start(self, dlpx_obj):
        """
        Starts a dataset
        :return:
        """
        svr_session = dlpx_obj.server_session
        try:
            if self.group:
                vdb_container_obj = ref.find_db_by_name_and_group(
                    svr_session, self.group, self.name
                )
                vdb_obj = ref.find_source_obj_by_containerref(
                    svr_session, source, vdb_container_obj.reference
                )
            else:
                vdb_obj = ref.find_obj_by_name(svr_session, source, self.name)
            source.start(svr_session, vdb_obj.reference)
            self._add_last_job_to_track(dlpx_obj)
        except (
            exceptions.RequestError,
            exceptions.JobError,
            AttributeError,
        ) as err:
            log.print_exception(
                f"An error occurred while starting dataset {self.name}: {err}"
            )
            # raise dxe.DlpxException(
            #     f"An error occurred while starting dataset {self.name}:
            #     {err}"
            # )
            self.failures[0] = True

    def _stop(self, dlpx_obj):
        """
        Stops a dataset
        :return:
        """
        svr_session = dlpx_obj.server_session
        try:
            if self.group:
                vdb_container_obj = ref.find_db_by_name_and_group(
                    svr_session, self.group, self.name
                )
                vdb_obj = ref.find_source_obj_by_containerref(
                    svr_session, source, vdb_container_obj.reference
                )
            else:
                vdb_obj = ref.find_obj_by_name(svr_session, source, self.name)
            source.stop(svr_session, vdb_obj.reference)
            self._add_last_job_to_track(dlpx_obj)
        except (
            exceptions.RequestError,
            exceptions.JobError,
            AttributeError,
        ) as err:
            log.print_exception(
                f"An error occurred while stopping dataset {self.name}: {err}"
            )
            # raise dxe.DlpxException(
            #     f"An error occurred while stopping dataset {self.name}:
            #     {err}"
            # )
            self.failures[0] = True

    def _enable(self, dlpx_obj):
        """
        Enables a dataset
        :return:
        """
        svr_session = dlpx_obj.server_session
        try:
            if self.group:
                vdb_container_obj = ref.find_db_by_name_and_group(
                    svr_session, self.group, self.name
                )
                vdb_obj = ref.find_source_obj_by_containerref(
                    svr_session, source, vdb_container_obj.reference
                )
            else:
                vdb_obj = ref.find_obj_by_name(svr_session, source, self.name)
            source.enable(svr_session, vdb_obj.reference)
            self._add_last_job_to_track(dlpx_obj)
        except (
            exceptions.RequestError,
            exceptions.JobError,
            AttributeError,
        ) as err:
            log.print_exception(
                f"An error occurred while enabling dataset {self.name}: {err}"
            )
            # raise dxe.DlpxException(
            #     f"An error occurred while enabling dataset {self.name}:
            #     {err}"
            # )
            self.failures[0] = True

    def _disable(self, dlpx_obj):
        """
        Disables a dataset
        :return:
        """
        svr_session = dlpx_obj.server_session
        disable_params = SourceDisableParameters()
        try:
            if self.group:
                vdb_container_obj = ref.find_db_by_name_and_group(
                    svr_session, self.group, self.name
                )
                vdb_obj = ref.find_source_obj_by_containerref(
                    svr_session, source, vdb_container_obj.reference
                )
            else:
                vdb_obj = ref.find_obj_by_name(svr_session, source, self.name)
            if self.force:
                disable_params.attempt_cleanup = False
                source.disable(svr_session, vdb_obj.reference, disable_params)
            else:
                source.disable(svr_session, vdb_obj.reference)
            self._add_last_job_to_track(dlpx_obj)
        except (
            exceptions.RequestError,
            exceptions.JobError,
            AttributeError,
        ) as err:
            log.print_exception(
                f"An error occurred while disabling dataset {self.name}: {err}"
            )
            # raise dxe.DlpxException(
            #     f"An error occurred while disabling dataset {self.name}:
            #     {err}"
            # )
            self.failures[0] = True

    def _list(self, dlpx_obj):
        """
        Lists all databases with stats for an engine
        """
        svr_session = dlpx_obj.server_session
        db_size = None
        active_space = None
        sync_space = None
        log_space = None
        try:
            all_source_objs = source.get_all(svr_session)
            all_consumer_objs = consumer.get_all(svr_session)
            for db_stats in all_consumer_objs:
                source_stats = ref.find_obj_list_by_containername(
                    all_source_objs, db_stats.container
                )
                if source_stats is not None:
                    active_space = (
                        db_stats.breakdown.active_space / 1024 / 1024 / 1024
                    )
                    sync_space = (
                        db_stats.breakdown.sync_space / 1024 / 1024 / 1024
                    )
                    log_space = db_stats.breakdown.log_space / 1024 / 1024
                    db_size = (
                        source_stats.runtime.database_size / 1024 / 1024 / 1024
                    )
                if source_stats.virtual is False:
                    print(
                        f"name: {db_stats.name}, ingest container:"
                        f" {db_stats.parent}, disk usage: {db_size:.2f}GB,"
                        f"Size of Snapshots: {active_space:.2f}GB, "
                        f"dSource Size: {sync_space:.2f}GB, "
                        f"Log Size: {log_space:.2f}MB,"
                        f"Enabled: {source_stats.runtime.enabled},"
                        f"Status: {source_stats.runtime.status}"
                    )
                elif source_stats.virtual is True:
                    print(
                        f"name: {db_stats.name}, ingest container: "
                        f"{db_stats.parent}, disk usage: "
                        f"{active_space:.2f}GB, Size of Snapshots: "
                        f"{sync_space:.2f}GB"
                        f"Log Size: {log_space:.2f}MB, Enabled: "
                        f"{source_stats.runtime.enabled}, "
                        f"Status: {source_stats.runtime.status}"
                    )
                elif source_stats is None:
                    print(
                        f"name: {db_stats.name},ingest container: "
                        f"{db_stats.parent}, database disk usage: "
                        f"{db_size:.2f}GB,"
                        f"Size of Snapshots: {active_space:.2f}GB,"
                        "Could not find source information. This could be a "
                        "result of an unlinked object"
                    )
        except (
            exceptions.RequestError,
            AttributeError,
            dxe.DlpxException,
        ) as err:
            log.print_exception(
                f"An error occurred while listing databases: {err}"
            )
            self.failures[0] = True
        except Exception as err:
            log.print_exception(
                f"An error occurred while listing databases: {err}"
            )
            self.failures[0] = True

    @run_async
    def _db_operation_helper(self, engine, dlpx_obj, single_thread):
        """
        This function is where we create our main workflow.
        Use the @run_async decorator to run this function asynchronously.
        The @run_async decorator to run multithreaded on Delphix Engines
        simultaneously
        :param engine: Dictionary of engines
        :type engine: dictionary
        :param dlpx_obj: DDP session object
        :type dlpx_obj: lib.GetSession.GetSession object
        :param single_thread: True - run single threaded,
                               False - run multi-thread
        :type single_thread: bool
        """
        dlpx_obj = self._initialize_session()
        self._setup_dlpx_session(dlpx_obj, engine)
        if dlpx_obj.server_session:
            try:
                with dlpx_obj.job_mode(single_thread):
                    if self.action == VirtualOps.START:
                        self._start(dlpx_obj)
                    elif self.action == VirtualOps.STOP:
                        self._stop(dlpx_obj)
                    elif self.action == VirtualOps.ENABLE:
                        self._enable(dlpx_obj)
                    elif self.action == VirtualOps.DISABLE:
                        self._disable(dlpx_obj)
                    elif self.action == VirtualOps.LIST:
                        self._list(dlpx_obj)
                    run_job.track_running_jobs(
                        engine,
                        dlpx_obj,
                        poll=self.poll,
                        failures=self.failures,
                    )
            except (
                dxe.DlpxException,
                exceptions.RequestError,
                exceptions.JobError,
                exceptions.HttpError,
            ) as err:
                log.print_exception(
                    f"Error in {basename(__file__)}:"
                    f'{engine["hostname"]}\n ERROR: {err}'
                )
                # raise err
                self.failures[0] = True



#
# Copyright (c) 2021 by Delphix. All rights reserved.
#

from delphixpy.v1_10_2 import exceptions
from delphixpy.v1_10_2.web import database
from delphixpy.v1_10_2.web import vo
from dxi._lib import dlpx_exceptions
from dxi._lib import dx_logging
from dxi._lib import dx_timeflow
from dxi._lib import get_references
from dxi._lib import run_job
from dxi._lib.run_async import run_async


class _RewindMixin(object):
    """
    Refresh a Delphix VDB
    """

    def _rewind_validate_data(self):
        dx_logging.print_debug("Validating inputs for rewind operation")
        if not self.vdb:
            raise Exception("Invalid VDB name")

        if self.time_stamp_type not in ["TIME", "SNAPSHOT"]:
            raise Exception("Either vdb name or group must be defined")

    @run_async
    def _rewind_helper(self, engine, dlpx_obj, single_thread):
        """
        This function is where we create our main workflow.
        Use the @run_async decorator to run this function asynchronously.
        The @run_async decorator allows us to run against multiple
        Delphix Engine simultaneously
        :param engine: Dictionary of engines
        :type engine: dictionary
        :param dlpx_obj: DDP session object
        :type dlpx_obj: lib.GetSession.GetSession object
        :param single_thread: True - run single threaded, False -
            run multi-thread
        :type single_thread: bool
        """
        func_ref = "_RewindMixin._rewind_helper()"
        self._rewind_validate_data()
        dlpx_obj = self._initialize_session()
        self._setup_dlpx_session(dlpx_obj, engine)
        try:
            dx_timeflow_obj = dx_timeflow.DxTimeflow(dlpx_obj.server_session)
            container_obj = get_references.find_obj_by_name(
                dlpx_obj.server_session, database, self.vdb
            )
            with dlpx_obj.job_mode(single_thread):
                dx_logging.print_debug(f" Rewinding {self.vdb}")
                container_obj = get_references.find_obj_by_name(
                    dlpx_obj.server_session, database, self.vdb
                )
                # Make sure our container object has a reference
                if container_obj.reference:
                    try:
                        if container_obj.runtime.enabled == "ENABLED":
                            dx_logging.print_debug(
                                f"INFO: {engine['hostname']} Rewinding "
                                f"{container_obj.name} to {self.time_stamp}\n"
                            )
                        elif (
                            container_obj.virtual is not True
                            or container_obj.staging is True
                        ):
                            raise dlpx_exceptions.DlpxException(
                                f"{container_obj.name} on "
                                f"Delphix Engine:{engine['hostname']} "
                                f"is not a virtual object. "
                                f"Rewind operation cannot continue.\n"
                            )
                    # This exception is raised if rewinding a vFiles VDB since
                    # AppDataContainer does not have virtual,
                    # staging or enabled attributes
                    except AttributeError:
                        pass
                    # If the vdb is a Oracle type, we need to use a
                    # OracleRollbackParameters
                    if str(container_obj.reference).startswith("ORACLE"):
                        rewind_params = vo.OracleRollbackParameters()
                    else:
                        rewind_params = vo.RollbackParameters()
                    rewind_params.timeflow_point_parameters = dx_timeflow_obj.set_timeflow_point(  # noqa
                        container_obj,
                        timestamp_type=self.time_stamp_type,
                        timestamp=self.time_stamp,
                        engine_session_ref=dlpx_obj.server_session,
                    )
                    try:
                        # Rewind the VDB
                        database.rollback(
                            dlpx_obj.server_session,
                            container_obj.reference,
                            rewind_params,
                        )
                    except (
                        exceptions.RequestError,
                        exceptions.HttpError,
                        exceptions.JobError,
                    ) as err:
                        raise dlpx_exceptions.DlpxException(
                            f"ERROR: {engine['hostname']} encountered "
                            f"an error on {container_obj.name} "
                            f"during the rewind process:\n{str(err)}",
                            func_ref,
                        )
                    self._add_last_job_to_track(dlpx_obj)
                # Don't do anything if the database is disabled
                else:
                    dx_logging.print_info(
                        f"{engine['hostname']}: {container_obj.name} is not "
                        f"enabled. Skipping sync."
                    )
                run_job.track_running_jobs(
                    engine, dlpx_obj, poll=self.poll, failures=self.failures
                )
        except (
            dlpx_exceptions.DlpxException,
            BaseException,
            Exception,
        ) as err:
            dx_logging.print_exception(err, func_ref)
            self.failures[0] = True



#
# Copyright (c) 2021 by Delphix. All rights reserved.
#
"""
This module includes classes for
dxi configuration related operations.
"""
import json
import os

from dxi._lib import dlpx_exceptions as dxe
from dxi._lib import dx_logging as log
from dxi._lib import util
from dxi._lib.util import DXIConfigConstantsHelper


class DXIConfigConstants(object):
    """
    Define constants for Config Operations.
    """

    CONFIG = DXIConfigConstantsHelper().get_config()
    LOG_DIR = DXIConfigConstantsHelper().get_logdir()
    MODULE_NAME = __name__


class DXIConfig:
    """
    Class for dxi config related opertaions.
    """

    def __init__(
        self,
        config=DXIConfigConstants.CONFIG,
        log_dir=DXIConfigConstants.LOG_DIR,
    ):
        """
        :param config: The path to the dxtools.conf file
        :type config: `str`

        :param log_dir: The path to the logfile for this module
        :type log_dir: `str`
        """
        self.module_name = DXIConfigConstants.MODULE_NAME
        self.key = ""
        self.config = config
        self.log_dir = log_dir

    def _encryption_helper(self):
        """
        Helper for encrypt operation
        """
        try:
            with open(self.config) as config_file:
                config = json.loads(config_file.read())
        except IOError as err:
            err_msg = (
                f"\nERROR: Unable to open {config_file}. Please "
                f"check the path, permissions and retry: {err}"
            )
            log.print_exception(err_msg)
            raise dxe.DlpxException(err_msg)
        except (ValueError, TypeError, AttributeError) as err:
            err_msg = (
                f"\nERROR: Unable to read {config_file} as json."
                f"Please check if the file is in a json format and retry:{err}"
            )
            log.print_exception(err_msg)
            raise dxe.DlpxException(err_msg)
        try:
            for each in config.keys():
                temp_config = config[each]
                temp_config["hostname"] = each
                self.key = util.get_encryption_key(temp_config["ip_address"])
                try:
                    if temp_config["encrypted"].lower() == "true":
                        isEncrypted = True
                    else:
                        isEncrypted = False
                except Exception:
                    isEncrypted = False
                if not isEncrypted:
                    temp_config["username"] = util.encrypt_data(
                        self.key, temp_config["username"]
                    )
                    temp_config["password"] = util.encrypt_data(
                        self.key, temp_config["password"]
                    )
                    temp_config["encrypted"] = "true"
                config[each] = temp_config
                with open(self.config, "w") as encrypt_file:
                    json.dump(config, encrypt_file, indent=4)
        except (Exception) as err:
            log.print_exception(
                f"Error: "
                f"There was an error while encrypting {self.config}\n {err}"
            )
            raise

    def encrypt(self):
        """
        Encrypt username and password information in the dxi config file.
        """
        try:
            self.config = util.find_config_path(self.config)
            self.log_dir = util.find_log_path(
                self.log_dir, DXIConfigConstants.MODULE_NAME
            )
            log.logging_est(self.log_dir)
            log.print_info(f"Starting encryption of file:{self.config}")
            self._encryption_helper()
            log.print_info(f"Encryption of file:{self.config} successful")
            return True
        except Exception:
            return False

    def init(self):
        """
        Initializes dxi configuration
            - Creats config directory and sample dxtools.conf file
            - Creates log directory if it does not exist.
        Raises:
          Exception: General Exception
        """
        try:
            if not self.config or not self.log_dir:
                raise Exception(
                    f"One or more required inputs were not provided.\n"
                    f"dxi init operation requires config and log directory locations.\n"  # noqa
                    f"Please check your input and retry."
                )
            return self._init_helper()
        except Exception as err:
            log.print_exception(f"Exception: {repr(err)}")
            return False

    def _init_helper(self):
        """
        Helper for dxi init operation

        Raises:
          Exception
        """
        init_complete = False
        try:
            util.init_global_config_folders(self.config, self.log_dir)
            init_complete = True
        except IOError as err:
            err_msg = (
                f"Error:  There was an error initializing dxi config"
                f"Reason: {err}"
            )
            log.print_exception(err_msg)
            raise dxe.DlpxException(err_msg)
        except Exception as err:
            err_msg = (
                f"Error:  There was an error initializing dxi config"
                f"Reason: {err}"
            )
            log.print_exception(err_msg)
            raise dxe.DlpxException(err_msg)
        finally:
            return init_complete


#
# Copyright (c) 2021 by Delphix. All rights reserved.
#

import click
from dxi._lib import click_util
from dxi._lib.util import boolean_based_system_exit
from dxi.bookmark.dxi_bookmark import BookmarkConstants
from dxi.bookmark.dxi_bookmark import DXIBookmark


@click.group(
    short_help="\b\nPerform Delphix Self-Service Boomark Operations.\n"
               "[create | delete | update | "
               "share | unshare | list]"
)
def bookmark():
    """
    Self-Service Bookmark operations
    """
    pass


# list
@bookmark.command()
@click.option("--tags", default=None, help="Tags to filter the bookmark names")
@click.option(
    "--engine",
    default=BookmarkConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread.\n"
    "[Default: {}]".format(BookmarkConstants.SINGLE_THREAD),
    default=BookmarkConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(BookmarkConstants.POLL),
    default=BookmarkConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=BookmarkConstants.PARALLEL,
)
@click.option(
    "--config",
    help="The path to the dxtools.conf file.",
    default=BookmarkConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="The path to the logfile you want to use.",
    default=BookmarkConstants.LOG_FILE_PATH,
)
def list(engine, single_thread, parallel, poll, config, log_file_path, tags):
    """
    List all self-service bookmarks on an engine.
    """
    bmk_obj = DXIBookmark(
        engine=engine,
        parallel=parallel,
        poll=poll,
        config=config,
        log_file_path=log_file_path,
        single_thread=single_thread,
    )
    boolean_based_system_exit(bmk_obj.list(tags=tags))


# share
@bookmark.command()
@click.option(
    "--bookmark-name",
    default=None,
    required=True,
    help="Name of the self-service bookmark.",
)
@click.option(
    "--engine",
    default=BookmarkConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread.\n"
    "[Default: {}]".format(BookmarkConstants.SINGLE_THREAD),
    default=BookmarkConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(BookmarkConstants.POLL),
    default=BookmarkConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=BookmarkConstants.PARALLEL,
)
@click.option(
    "--config",
    help="The path to the dxtools.conf file.",
    default=BookmarkConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="The path to the logfile you want to use.",
    default=BookmarkConstants.LOG_FILE_PATH,
)
def share(
    bookmark_name, engine, single_thread, parallel, poll, config, log_file_path
):
    """
    Share a self-service bookmark.
    """
    bmk_obj = DXIBookmark(
        engine=engine,
        parallel=parallel,
        poll=poll,
        config=config,
        log_file_path=log_file_path,
        single_thread=single_thread,
    )
    boolean_based_system_exit(bmk_obj.share(bookmark_name=bookmark_name))


# unshare
@bookmark.command()
@click.option(
    "--bookmark-name",
    default=None,
    required=True,
    help="Name of the self-service bookmark.",
)
@click.option(
    "--engine",
    default=BookmarkConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread.\n"
    "[Default: {}]".format(BookmarkConstants.SINGLE_THREAD),
    default=BookmarkConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(BookmarkConstants.POLL),
    default=BookmarkConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=BookmarkConstants.PARALLEL,
)
@click.option(
    "--config",
    help="The path to the dxtools.conf file.",
    default=BookmarkConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="The path to the logfile you want to use.",
    default=BookmarkConstants.LOG_FILE_PATH,
)
def unshare(
    bookmark_name, engine, single_thread, parallel, poll, config, log_file_path
):
    """
    Unshare a self-service bookmark.
    """
    bmk_obj = DXIBookmark(
        engine=engine,
        parallel=parallel,
        poll=poll,
        config=config,
        log_file_path=log_file_path,
        single_thread=single_thread,
    )
    boolean_based_system_exit(bmk_obj.unshare(bookmark_name=bookmark_name))


# delete
@bookmark.command()
@click.option(
    "--bookmark-name",
    default=None,
    required=True,
    help="Name of the self-service bookmark.",
)
@click.option(
    "--engine",
    default=BookmarkConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread.\n"
    "[Default: {}]".format(BookmarkConstants.SINGLE_THREAD),
    default=BookmarkConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(BookmarkConstants.POLL),
    default=BookmarkConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=BookmarkConstants.PARALLEL,
)
@click.option(
    "--config",
    help="The path to the dxtools.conf file.",
    default=BookmarkConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="The path to the logfile you want to use.",
    default=BookmarkConstants.LOG_FILE_PATH,
)
def delete(
    bookmarkname, engine, single_thread, parallel, poll, config, log_file_path
):
    """
    Delete a self-service bookmark.
    """
    bmk_obj = DXIBookmark(
        engine=engine,
        parallel=parallel,
        poll=poll,
        config=config,
        log_file_path=log_file_path,
        single_thread=single_thread,
    )
    boolean_based_system_exit(bmk_obj.delete(bookmark_name=bookmarkname))


# update
@bookmark.command()
@click.option(
    "--bookmark-name",
    default=None,
    required=True,
    help="Name of the self-service bookmark.",
)
@click.option(
    "--new-name",
    default=None,
    help="If updating bookmark name, provide the new name.",
)
@click.option(
    "--tags",
    default=None,
    help="\b\nIf updating tags, provide new tags.\n"
    "All existing tags on the bookmark will be \n"
    "replaced with new tags provided.",
)
@click.option(
    "--description",
    default=None,
    help="If updating bookmark description, provide the new description.",
)
@click.option(
    "--expires",
    default=None,
    help="If updating bookmark expiration, provide new expiration date-time."
    'Format: "%Y-%m-%dT%H:%M:%S" ',
)
@click.option(
    "--engine",
    default=BookmarkConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread.\n"
    "[Default: {}]".format(BookmarkConstants.SINGLE_THREAD),
    default=BookmarkConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(BookmarkConstants.POLL),
    default=BookmarkConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=BookmarkConstants.PARALLEL,
)
@click.option(
    "--config",
    help="The path to the dxtools.conf file.",
    default=BookmarkConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="The path to the logfile you want to use.",
    default=BookmarkConstants.LOG_FILE_PATH,
)
def update(
    bookmark_name,
    new_name,
    tags,
    description,
    expires,
    engine,
    single_thread,
    parallel,
    poll,
    config,
    log_file_path,
):
    """
    Update a self-service bookmark.
    """
    bmk_obj = DXIBookmark(
        engine=engine,
        parallel=parallel,
        poll=poll,
        config=config,
        log_file_path=log_file_path,
        single_thread=single_thread,
    )
    boolean_based_system_exit(
        bmk_obj.update(
            bookmark_name=bookmark_name,
            new_bookmark_name=new_name,
            tags=tags,
            description=description,
            expires=expires,
        )
    )


# create
@bookmark.command()
@click.option(
    "--bookmark-name",
    default=None,
    required=True,
    help="Name of the self-service bookmark.",
)
@click.option(
    "--container-name",
    cls=click_util.MutuallyExclusiveOption,
    mutually_exclusive=["template-name"],
    default=None,
    help="\b\nName of the parent self-service container for the bookmark.\n"
    "[required: If bookmark is being created on a self-service container]",
)
@click.option(
    "--template-name",
    cls=click_util.MutuallyExclusiveOption,
    mutually_exclusive=["containername"],
    default=None,
    help="\b\nName of the self-service parent template for the bookmark.\n"
    "[required: If bookmark is being created on a self-service template]",
)
@click.option(
    "--branch-name",
    default=None,
    help="\b\nIf self-service bookmark is not unique in a container,\n"
    "specify a branch name to create the bookmark from,",
)
@click.option(
    "--timestamp",
    default=None,
    help="\b\nTimestamp to create the self-service bookmark. \n"
    "Timestamp should be in the same timezone as the Delphix Engine.\n"
    'Format: "%Y-%m-%dT%H:%M:%S" | latest \n'
    "[Example: 2021-12-21T16:22:77]",
)
@click.option(
    "--expires",
    default=None,
    help="\b\nSet self-service bookmark expiration time. \n"
    "Expiration time should be in the same timezone as the Delphix Engine.\n"
    'Format: "%Y-%m-%dT%H:%M:%S" | latest \n'
    "[Example: 2021-12-21T16:22:77]",
)
@click.option(
    "--tags", default=None, help="Tags to set on the self-service bookmark."
)
@click.option(
    "--description",
    default=None,
    help="Description for the self-service bookmark.",
)
@click.option(
    "--engine",
    default=BookmarkConstants.ENGINE_ID,
    help="\b\nName of Delphix Engine in the config file.\n"
    "[Default: default]",
)
@click.option(
    "--single-thread",
    help="\b\nRun as a single thread.\n"
    "[Default: {}]".format(BookmarkConstants.SINGLE_THREAD),
    default=BookmarkConstants.SINGLE_THREAD,
    is_flag=True,
)
@click.option(
    "--poll",
    type=click.INT,
    help="\b\nThe number of seconds to wait between job polls.\n"
    "[Default: {}s]".format(BookmarkConstants.POLL),
    default=BookmarkConstants.POLL,
)
@click.option(
    "--parallel",
    type=click.INT,
    help="Limit number of jobs to maxjob.",
    default=BookmarkConstants.PARALLEL,
)
@click.option(
    "--config",
    help="The path to the dxtools.conf file.",
    default=BookmarkConstants.CONFIG,
)
@click.option(
    "--log-file-path",
    help="The path to the logfile you want to use.",
    default=BookmarkConstants.LOG_FILE_PATH,
)
def create(
    bookmark_name,
    container_name,
    template_name,
    branch_name,
    timestamp,
    expires,
    tags,
    description,
    engine,
    single_thread,
    parallel,
    poll,
    config,
    log_file_path,
):
    """
    Create a new self-service bookmark.
    """
    bmk_obj = DXIBookmark(
        engine=engine,
        parallel=parallel,
        poll=poll,
        config=config,
        log_file_path=log_file_path,
        single_thread=single_thread,
    )
    boolean_based_system_exit(
        bmk_obj.create(
            bookmark_name=bookmark_name,
            container_name=container_name,
            template_name=template_name,
            branch_name=branch_name,
            timestamp=timestamp,
            expires=expires,
            tags=tags,
            description=description,
        )
    )


if __name__ == "__main__":
    create()



#
# Copyright (c) 2021 by Delphix. All rights reserved.
#
"""
Create an object to link MS SQL or ASE dSources
"""
from delphixpy.v1_10_2.web import group
from delphixpy.v1_10_2.web import sourceconfig
from delphixpy.v1_10_2.web import vo
from delphixpy.v1_10_2.web.environment import environment

from dxi._lib import dlpx_exceptions as dxe, get_references
from dxi._lib import get_references as ref
from dxi._lib.util import DXIConfigConstantsHelper
from dxi.dxi_tool_base import DXIBase


class ProvisionDsourceConstants(object):
    """
    Class of common constants used by Provision VDB
    """

    SINGLE_THREAD = False
    POLL = 10
    CONFIG = DXIConfigConstantsHelper().get_config()
    LOG_FILE_PATH = DXIConfigConstantsHelper().get_logdir()
    ENGINE_ID = "default"
    PARALLEL = 5
    ACTION = None
    MODULE_NAME = __name__
    VDB_LIST_HEADER = []
    FORCE = False


class DsourceLink:
    """
    Base class for linking dSources
    """

    def __init__(
        self,
        dsource_name,
        db_passwd,
        db_user,
        dx_group,
        logsync,
        dlpx_obj,
        db_type,
    ):
        """
        Attributes required for linking dSources
        :param engine: A Delphix DDP session object
        :type engine: lib.get_session.GetSession
        :param dsource_name: Name of the dsource
        :type dsource_name: str
        :param dx_group: Group name of where the dSource will reside
        :type dx_group: str
        :param db_passwd: Password of the db_user
        :type db_passwd: str
        :param db_user: Username of the dSource
        :type db_user: str
        :param db_type: dSource type. mssql, sybase or oracle
        :type db_type: str
        """
        self.dx_group = dx_group
        self.db_passwd = db_passwd
        self.db_user = db_user
        self.dsource_name = dsource_name
        self.db_type = db_type
        self.dlpx_obj = dlpx_obj
        self.logsync = logsync
        self.link_params = vo.LinkParameters()
        self.srccfg_obj = self._get_source_config_type(self.db_type)

    def _get_source_config_type(self, db_type):
        return ""

    def dsource_prepare_link(self, source_env_name=None):
        """
        Prepare the dsource object for linking
        """
        self.link_params.name = self.dsource_name
        if self.db_type.lower() == "oracle":
            self.link_params.link_data = vo.OracleLinkData()
        elif self.db_type.lower() == "sybase":
            self.link_params.link_data = vo.ASELinkData()
        elif self.db_type.lower() == "mssql":
            self.link_params.link_data = vo.MSSqlLinkData()
        self.link_params.group = ref.find_obj_by_name(
            self.dlpx_obj.server_session, group, self.dx_group
        ).reference
        self.link_params.link_data.db_credentials = vo.PasswordCredential()
        self.link_params.link_data.db_credentials.password = self.db_passwd
        self.link_params.link_data.db_user = self.db_user
        # Create blank sourcing policy
        self.link_params.link_data.sourcing_policy = vo.SourcingPolicy()
        self.link_params.link_data.sourcing_policy.logsync_enabled = False
        self.link_params.link_data.config = self.get_or_create_sourceconfig(
            self.srccfg_obj, source_env_name
        )

        return self.link_params

    def get_or_create_sourceconfig(self, sourceconfig_obj=None, source_env_name=None):
        """
        Get current sourceconfig or create it
        :param sourceconfig_obj:
        :return: link_params
        """
        try:
            if self.db_type.lower() == "mssql":
                env_ref = get_references.find_obj_by_name(
                    self.dlpx_obj.server_session, environment, source_env_name)
                return ref.find_obj_by_name_v2(
                    self.dlpx_obj.server_session, sourceconfig, self.dsource_name, environment=env_ref.reference
                ).reference
            else:
                return ref.find_obj_by_name(
                    self.dlpx_obj.server_session, sourceconfig, self.dsource_name
                ).reference
        except dxe.DlpxObjectNotFound:
            self.link_params.link_data.config = sourceconfig.create(
                self.dlpx_obj.server_session, sourceconfig_obj
            ).reference
